/*---------------- Search Box */

#MSearchBox {
    white-space : nowrap;
    background: white;
    border-radius: 0.65em;
    box-shadow: inset 0.5px 0.5px 3px 0px #555;
    z-index: 102;
}

#MSearchBox .left {
    display: inline-block;
    vertical-align: middle;
    height: 1.4em;
}

#MSearchSelect {
    display: inline-block;
    vertical-align: middle;
    height: 19px;
    padding: 0 0 0 0.3em;
    margin: 0;
}

#MSearchField {
    display: inline-block;
    vertical-align: middle;
    width: 7.5em;
    height: 19px;
    margin: 0 0.15em;
    padding: 0;
    line-height: 1em;
    border:none;
    color: #909090;
    outline: none;
    font-family: Arial, Verdana, sans-serif;
    -webkit-border-radius: 0px;
    border-radius: 0px;
    background: none;
}

@media(hover: none) {
    /* to avoid zooming on iOS */
    #MSearchField {
        font-size: 16px;
    }
}

#MSearchBox .right {
    display: inline-block;
    vertical-align: middle;
    width: 1.4em;
    height: 1.4em;
}

#MSearchClose {
    display: none;
    font-size: inherit;
    background : none;
    border: none;
    margin: 0;
    padding: 0;
    outline: none;

}

#MSearchCloseImg {
    height: 1.4em;
    padding: 0.3em;
    margin: 0;
}

.MSearchBoxActive #MSearchField {
    color: #000000;
}

#main-menu > li:last-child {
    /* This <li> object is the parent of the search bar */
    display: flex;
    justify-content: center;
    align-items: center;
    height: 36px;
    margin-right: 1em;
}

